/**This class constructs the SFOC vector.
 * 12 points are given as input and the vector is created by linearizing between
 * these points.
 * 
 * @author Andreas Carlsen
 *
 */
public class SFOCVector {
	private double[][] vec;
	
	/**Constructor that creates the SFOC vector from 12 points.
	 * The input vector contains twelve points from each generator.
	 * The points are the SFOC for 25%, 40%, 50%, 56%, 62%, 68%, 75%, 80%, 85%, 
	 * 90%, 100% and 110% load.
	 * 
	 * @param SFOC_meas		Input vector containing 12 SFOC points for each generator
	 */
	public SFOCVector(double[][] SFOC_meas){
		vec = new double[4][111];
		vec[0][25]=SFOC_meas[0][0];
		vec[1][25]=SFOC_meas[1][0];
		vec[2][25]=SFOC_meas[2][0];
		vec[3][25]=SFOC_meas[3][0];
		
		for(int i=1; i<25;i++){
			vec[0][i] = 99999999;
			vec[1][i] = 99999999;
			vec[2][i] = 99999999;
			vec[3][i] = 99999999;
		}
		
		vec[0][25] = SFOC_meas[0][0];
		vec[1][25] = SFOC_meas[1][0];
		vec[2][25] = SFOC_meas[2][0];
		vec[3][25] = SFOC_meas[3][0];
		
		for(int i=26; i<41;i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][1]-SFOC_meas[0][0])/15;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][1]-SFOC_meas[1][0])/15;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][1]-SFOC_meas[2][0])/15;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][1]-SFOC_meas[3][0])/15;
		}
		
		for(int i=41; i<51; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][2]-SFOC_meas[0][1])/10;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][2]-SFOC_meas[1][1])/10;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][2]-SFOC_meas[2][1])/10;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][2]-SFOC_meas[3][1])/10;
		}
		
		for(int i=51; i<57; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][3]-SFOC_meas[0][2])/6;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][3]-SFOC_meas[1][2])/6;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][3]-SFOC_meas[2][2])/6;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][3]-SFOC_meas[3][2])/6;
		}
		
		for(int i=57; i<63; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][4]-SFOC_meas[0][3])/6;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][4]-SFOC_meas[1][3])/6;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][4]-SFOC_meas[2][3])/6;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][4]-SFOC_meas[3][3])/6;
		}
		
		for(int i=63; i<69; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][5]-SFOC_meas[0][4])/6;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][5]-SFOC_meas[1][4])/6;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][5]-SFOC_meas[2][4])/6;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][5]-SFOC_meas[3][4])/6;
		}
		
		for(int i=69; i<76; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][6]-SFOC_meas[0][5])/7;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][6]-SFOC_meas[1][5])/7;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][6]-SFOC_meas[2][5])/7;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][6]-SFOC_meas[3][5])/7;
		}
		
		for(int i=76; i<81; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][7]-SFOC_meas[0][6])/5;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][7]-SFOC_meas[1][6])/5;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][7]-SFOC_meas[2][6])/5;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][7]-SFOC_meas[3][6])/5;
		}
		
		for(int i=81; i<86; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][8]-SFOC_meas[0][7])/5;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][8]-SFOC_meas[1][7])/5;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][8]-SFOC_meas[2][7])/5;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][8]-SFOC_meas[3][7])/5;
		}
		
		for(int i=86; i<91; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][9]-SFOC_meas[0][8])/5;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][9]-SFOC_meas[1][8])/5;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][9]-SFOC_meas[2][8])/5;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][9]-SFOC_meas[3][8])/5;
		}
		
		for(int i=91; i<101; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][10]-SFOC_meas[0][9])/10;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][10]-SFOC_meas[1][9])/10;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][10]-SFOC_meas[2][9])/10;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][10]-SFOC_meas[3][9])/10;
		}
		
		for(int i=101; i<111; i++){
			vec[0][i] = vec[0][i-1]+(SFOC_meas[0][11]-SFOC_meas[0][10])/10;
			vec[1][i] = vec[1][i-1]+(SFOC_meas[1][11]-SFOC_meas[1][10])/10;
			vec[2][i] = vec[2][i-1]+(SFOC_meas[2][11]-SFOC_meas[2][10])/10;
			vec[3][i] = vec[3][i-1]+(SFOC_meas[3][11]-SFOC_meas[3][10])/10;
		}
	}
	
	/**Returns the SFOC vector
	 * 
	 * @return					SFOC vector (vec)
	 */
	public double[][] getVec() {
		return vec;
	}
	
	/**Returns the SFOC of a given load. The load is given by the index parameter and 
	 * the generator is given by the gen parameter
	 * 
	 * @param index				The load of the generator
	 * @param gen				Number representing the generator
	 * @return					The SFOC of the given load and generator
	 */
	public double getValueOfIndex(double index, int gen){
		int low_index = (int) index;
		int high_index = (int) Math.ceil(index);
		if(low_index != high_index){
			double diff = index-low_index;
			double hl_diff = this.vec[gen][high_index]-this.vec[gen][low_index];
			return diff*hl_diff + this.vec[gen][low_index];
		}else{
			return this.vec[gen][low_index];
		}
	}
	
}
