/**This class is created for use with the simple simulator.
 * It contains methods for setting the load load set points of the generators in a 
 * balanced setting.
 * 
 * @author Andreas Carlsen
 *
 */
public class balancedLoadSetPoints {

	private int nr_of_gen;
	private double[] w_max;
	private double[] up_limit;
	private double demand;
	
	/**Constructor that sets the variables
	 * 
	 * @param w_max				Contains the maximum power for each generator
	 */
	public balancedLoadSetPoints(double[] w_max, double[] up_limit){
		this.nr_of_gen = w_max.length;
		this.w_max = w_max;
		this.up_limit = up_limit;
	}

	/**Returns the maximum power for each generator
	 * 
	 * @return					The maximum power for each generator (w_max)
	 */
	public double[] getW_max() {
		return w_max;
	}

	/**Sets the power demand
	 * 
	 * @param demand			Power demand
	 */
	public void setDemand(double demand) {
		this.demand = demand;
	}
	
	/**interface to the optimal load algorithm
	 */
	public double[] calculateOptimalLoad(){
		return  OptimalLoad();
	}

	/**Contains the optimal load algorithm
	 * 
	 * @return					Vector containing the optimal load for each generator (load)
	 */
	public double[] OptimalLoad(){
		double[] load = new double[this.nr_of_gen];
		if(this.demand<=((this.up_limit[0]*this.w_max[0]/100))){
			load[0]=(this.demand*100)/(this.w_max[0]);
			return load;
		}else if(this.demand<=((this.up_limit[0]*this.w_max[0]/100)+(this.up_limit[1]*this.w_max[1]/100))){
			load[0]=(this.demand*100)/(this.w_max[0]+this.w_max[1]);
			load[1]=(this.demand*100)/(this.w_max[0]+this.w_max[1]);
			return load;
		}else if(this.demand<=((this.up_limit[0]*this.w_max[0]/100)+(this.up_limit[1]*this.w_max[1]/100)
				+(this.up_limit[2]*this.w_max[2])/100)){
			load[0]=(this.demand*100)/(this.w_max[0]+this.w_max[1]+this.w_max[2]);
			load[1]=(this.demand*100)/(this.w_max[0]+this.w_max[1]+this.w_max[2]);
			load[2]=(this.demand*100)/(this.w_max[0]+this.w_max[1]+this.w_max[2]);
			return load;
		}else if(this.demand<=((this.up_limit[0]*this.w_max[0]/100)+(this.up_limit[1]*this.w_max[1]/100)
				+(this.up_limit[2]*this.w_max[2]/100)+(this.up_limit[3]*this.w_max[3]/100))){
			load[0]=(this.demand*100)/(this.w_max[0]+this.w_max[1]+this.w_max[2]+this.w_max[3]);
			load[1]=(this.demand*100)/(this.w_max[0]+this.w_max[1]+this.w_max[2]+this.w_max[3]);
			load[2]=(this.demand*100)/(this.w_max[0]+this.w_max[1]+this.w_max[2]+this.w_max[3]);
			load[3]=(this.demand*100)/(this.w_max[0]+this.w_max[1]+this.w_max[2]+this.w_max[3]);
			return load;
		}
		
		return load;
	}
	
}
