/**This class is created for use with the simple simulator.
 * It contains a simple algorithm for determining the number of running
 * generators. This is not the same algorithm as in the Metso DNA simulator,
 * but the high limits of the generators can be adjusted to create the same 
 * situations.
 * 
 * @author Andreas Carlsen
 *
 */
public class generatorChooser {
	private double[] high_limit = new double[4];
	private int[] bin_var = new int[4];
	
	/**Constructor that sets the high limits.
	 * 
	 * @param limit				Contains the hight limits for the different generators.
	 */
	public generatorChooser(double[] limit){
		high_limit=limit;
	}

	/**Returns the SFOC vector
	 * 
	 * @param generators			Contains the maximum power of each generator
	 * @param demand				Power demand
	 * @param gen_pris				Contains generator numbers for each priority
	 * @return						A vector containing a running bit for each generator (bin_var)
	 */
	public int[] setRunningGens(double[] generators, double demand, int[]gen_pris){
		bin_var[gen_pris[0]]=0;
		bin_var[gen_pris[1]]=0;
		bin_var[gen_pris[2]]=0;
		bin_var[gen_pris[3]]=0;
		if(demand<=((this.high_limit[0]*generators[gen_pris[0]]/100))){
			bin_var[gen_pris[0]]=1;
			return bin_var;
		}else if(demand<=((this.high_limit[0]*generators[gen_pris[0]]/100)+(this.high_limit[1]*generators[gen_pris[1]]/100))){
			bin_var[gen_pris[0]]=1;
			bin_var[gen_pris[1]]=1;
			return bin_var;
		}else if(demand<=((this.high_limit[0]*generators[gen_pris[0]]/100)+(this.high_limit[1]*generators[gen_pris[1]]/100)
				+(this.high_limit[2]*generators[gen_pris[2]])/100)){
			bin_var[gen_pris[0]]=1;
			bin_var[gen_pris[1]]=1;
			bin_var[gen_pris[2]]=1;
			return bin_var;
		}else if(demand<=((this.high_limit[0]*generators[gen_pris[0]]/100)+(this.high_limit[1]*generators[gen_pris[1]]/100)
				+(this.high_limit[2]*generators[gen_pris[2]]/100)+(this.high_limit[3]*generators[gen_pris[3]]/100))){
			bin_var[gen_pris[0]]=1;
			bin_var[gen_pris[1]]=1;
			bin_var[gen_pris[2]]=1;
			bin_var[gen_pris[3]]=1;
			return bin_var;
		}
		return bin_var;
	}
}
