import java.io.FileNotFoundException;
import java.text.ParseException;
import Valmet.AITypes.*;

/**
 * Java source code for the unbalLoadOpt module. This module sets the optimal
 * load set points for each generator.
 * 
 * @author Andreas Carlsen
 *
 */

public class unbalLoadOpt
{
	int nr_of_gens = 4;
	int nr_of_points = 12;
	Ana demand;
	Bin	new_data;
	Bin dg1_bin;
	Ints dg1_pri;
	Ana dg1_hl;
	Ana dg1_ll;
	Ana dg1_np;
	AnaVector dg1_p;
	Bin dg2_bin;
	Ints dg2_pri;
	Ana dg2_hl;
	Ana dg2_ll;
	Ana dg2_np;
	AnaVector dg2_p;
	Bin dg3_bin;
	Ints dg3_pri;
	Ana dg3_hl;
	Ana dg3_ll;
	Ana dg3_np;
	AnaVector dg3_p;
	Bin dg4_bin;
	Ints dg4_pri;
	Ana dg4_hl;
	Ana dg4_ll;
	Ana dg4_np;
	AnaVector dg4_p;
	
	Ana dg1_sp;
	Ana dg2_sp;
	Ana dg3_sp;
	Ana dg4_sp;
	
	double[] w_max;
	int[] gen_bins;
	int[] gen_pris;
	double[][] SFOC_meas;
	double[][] SFOC_complete;
	double[] set_points;
	double[] hl;
	double[] ll;
	unbalancedLoadSetPoints model;
	SFOCVector svec;

	/**
	 * Constructor
	 * Creates parameter objects and connects them 
	 * to the inputs and outputs of the prog2 function block.
	 *
	 */
	public unbalLoadOpt() throws AITableReferenceException {

		// Connect member variables to the prog2 inputs/outputs.
		try 
		{
			demand = new Ana("demand");
			new_data = new Bin("new_data");
			dg1_bin = new Bin("dg1_bin");
			dg1_pri = new Ints("dg1_pri");
			dg1_hl = new Ana("dg1_hl");
			dg1_ll = new Ana("dg1_ll");
			dg1_np = new Ana("dg1_np");
			dg1_p = new AnaVector("dg1_p");
			dg2_bin = new Bin("dg2_bin");
			dg2_pri = new Ints("dg2_pri");
			dg2_hl = new Ana("dg2_hl");
			dg2_ll = new Ana("dg2_ll");
			dg2_np = new Ana("dg2_np");
			dg2_p = new AnaVector("dg2_p");
			dg3_bin = new Bin("dg3_bin");
			dg3_pri = new Ints("dg3_pri");
			dg3_hl = new Ana("dg3_hl");
			dg3_ll = new Ana("dg3_ll");
			dg3_np = new Ana("dg3_np");
			dg3_p = new AnaVector("dg3_p");
			dg4_bin = new Bin("dg4_bin");
			dg4_pri = new Ints("dg4_pri");
			dg4_hl = new Ana("dg4_hl");
			dg4_ll = new Ana("dg4_ll");
			dg4_np = new Ana("dg4_np");
			dg4_p = new AnaVector("dg4_p");
			
			dg1_sp = new Ana("dg1_sp");
			dg2_sp = new Ana("dg2_sp");
			dg3_sp = new Ana("dg3_sp");
			dg4_sp = new Ana("dg4_sp");
			
		}
		catch (Throwable e)
		{
			throw new AINullTablePointerException("HVACFlagSetter: initialization failed: "
					+ e.getMessage());
		}


		// Check that table connections are ok.
		checkInitialTableConnections();
	}

	public static void main (String args[]) throws ParseException, AITableReferenceException, FileNotFoundException{
		
	}

	/**
	 * checkInitialTableConnections
	 * Checks that parameter tables are properly connected.
	 */
	private final void checkInitialTableConnections() throws AITableReferenceException
	{ 

	 	try {
		  dg1_p.checkGenConnected();
		}
		catch (AITableReferenceException e) {
			throw new AITableGenCheckException("unbalLoadOpt/dg1_p: " + e.getMessage());
		}
		try {
		  dg2_p.checkGenConnected();
		}
		catch (AITableReferenceException e) {
			throw new AITableGenCheckException("unbalLoadOpt/dg2_p: " + e.getMessage());
		}
		try {
		  dg3_p.checkGenConnected();
		}
		catch (AITableReferenceException e) {
			throw new AITableGenCheckException("unbalLoadOpt/dg3_p: " + e.getMessage());
		}
		try {
		  dg4_p.checkGenConnected();
		}
		catch (AITableReferenceException e) {
			throw new AITableGenCheckException("unbalLoadOpt/dg4_p: " + e.getMessage());
		}

	}

	/**
	 * checkTableConnections
	 * Checks the integrity of the parameter tables.
	 */ 
	private final void checkTableConnections() throws AITableReferenceException
	{ 
		try {
			 dg1_p.checkTableFrame();
			 dg1_p.checkInputTable();
		}
		catch (AITableReferenceException e) {
			 throw new AITableGenCheckException("unbalLoadOpt/dg1_p:" + e.getMessage());
		}
		try {
			 dg2_p.checkTableFrame();
			 dg2_p.checkInputTable();
		}
		catch (AITableReferenceException e) {
			 throw new AITableGenCheckException("unbalLoadOpt/dg2_p:" + e.getMessage());
		}
		try {
			 dg3_p.checkTableFrame();
			 dg3_p.checkInputTable();
		}
		catch (AITableReferenceException e) {
			 throw new AITableGenCheckException("unbalLoadOpt/dg3_p:" + e.getMessage());
		}
		try {
			 dg4_p.checkTableFrame();
			 dg4_p.checkInputTable();
		}
		catch (AITableReferenceException e) {
			 throw new AITableGenCheckException("unbalLoadOpt/dg4_p:" + e.getMessage());
		} 

	}

	/**
	 * init
	 * This method is called once for each prog2 function block
	 * using this class. The method is called when the function
	 * block is loaded to the application server.
	 */
	public void init() {
		
		w_max = new double[nr_of_gens];		
		set_points = new double[nr_of_gens];
		gen_bins = new int[nr_of_gens];
		SFOC_meas = new double[nr_of_gens][nr_of_points];
		gen_pris = new int[nr_of_gens];
		hl = new double[nr_of_gens];
		ll = new double[nr_of_gens];
		
		SFOC_meas[0][0] = 226.3;
		SFOC_meas[0][1] = 199.8;
		SFOC_meas[0][2] = 190.6;
		SFOC_meas[0][3] = 189;
		SFOC_meas[0][4] = 185.8;
		SFOC_meas[0][5] = 188.8;
		SFOC_meas[0][6] = 184.3;
		SFOC_meas[0][7] = 181.5;
		SFOC_meas[0][8] = 179;
		SFOC_meas[0][9] = 182.5;
		SFOC_meas[0][10] = 183.8;
		SFOC_meas[0][11] = 186.8;
		SFOC_meas[1][0] = 226.4;
		SFOC_meas[1][1] = 205.64;
		SFOC_meas[1][2] = 191.8;
		SFOC_meas[1][3] = 190.672;
		SFOC_meas[1][4] = 188.744;
		SFOC_meas[1][5] = 188.416;
		SFOC_meas[1][6] = 187.1;
		SFOC_meas[1][7] = 183.75;
		SFOC_meas[1][8] = 180.4;
		SFOC_meas[1][9] = 183.45;
		SFOC_meas[1][10] = 186.5;
		SFOC_meas[1][11] = 188.2;	
		SFOC_meas[2][0] = 224.6;
		SFOC_meas[2][1] = 198;
		SFOC_meas[2][2] = 191.2;
		SFOC_meas[2][3] = 187.6;
		SFOC_meas[2][4] = 186.6;
		SFOC_meas[2][5] = 188.3;
		SFOC_meas[2][6] = 184.8;
		SFOC_meas[2][7] = 181.9;
		SFOC_meas[2][8] = 180.7;
		SFOC_meas[2][9] = 183.2;
		SFOC_meas[2][10] = 184.1;
		SFOC_meas[2][11] = 187.9;
		SFOC_meas[3][0] = 225;
		SFOC_meas[3][1] = 202.86;
		SFOC_meas[3][2] = 188.1;
		SFOC_meas[3][3] = 187.548;
		SFOC_meas[3][4] = 186.996;
		SFOC_meas[3][5] = 186.444;
		SFOC_meas[3][6] = 185.8;
		SFOC_meas[3][7] = 183.5;
		SFOC_meas[3][8] = 181.2;
		SFOC_meas[3][9] = 182.2;
		SFOC_meas[3][10] = 184.2;
		SFOC_meas[3][11] = 187.6;
	} // End of init

	/**
	 * run 
	 * This method is called when the prog2 function block is executed.
	 */
	public void run() {

		try {
			checkTableConnections();
			runMe();
		}
		catch (Throwable e) {
			System.err.println("HVACFlagSetter: " + e.getMessage());
		}

	} // End of run

	/**
	 * runMe 
	 * This method is called from method run().
	 * @throws AIFailsException 
	 * @throws AITableReferenceException 
	 */     
	private void runMe() throws AIFailsException, AITableReferenceException
	{	
		gen_bins[0] = dg1_bin.getIntValue();
		gen_bins[1] = dg2_bin.getIntValue();
		gen_bins[2] = dg3_bin.getIntValue();
		gen_bins[3] = dg4_bin.getIntValue();
		
		gen_pris[dg1_pri.getIntValue()-1] = 0;
		gen_pris[dg2_pri.getIntValue()-1] = 1;
		gen_pris[dg3_pri.getIntValue()-1] = 2;
		gen_pris[dg4_pri.getIntValue()-1] = 3;
		
		hl[0] = dg1_hl.getDoubleValue();
		hl[1] = dg2_hl.getDoubleValue();
		hl[2] = dg3_hl.getDoubleValue();
		hl[3] = dg4_hl.getDoubleValue();
		
		ll[0] = dg1_ll.getDoubleValue();
		ll[1] = dg2_ll.getDoubleValue();
		ll[2] = dg3_ll.getDoubleValue();
		ll[3] = dg4_ll.getDoubleValue();
		
		
		if(new_data.getBooleanValue()){
			SFOC_meas[0][0] = dg1_p.getElem(0).getDoubleValue();
			SFOC_meas[0][1] = dg1_p.getElem(1).getDoubleValue();
			SFOC_meas[0][2] = dg1_p.getElem(2).getDoubleValue();
			SFOC_meas[0][3] = dg1_p.getElem(3).getDoubleValue();
			SFOC_meas[0][4] = dg1_p.getElem(4).getDoubleValue();
			SFOC_meas[0][5] = dg1_p.getElem(5).getDoubleValue();
			SFOC_meas[0][6] = dg1_p.getElem(6).getDoubleValue();
			SFOC_meas[0][7] = dg1_p.getElem(7).getDoubleValue();
			SFOC_meas[0][8] = dg1_p.getElem(8).getDoubleValue();
			SFOC_meas[0][9] = dg1_p.getElem(9).getDoubleValue();
			SFOC_meas[0][10] = dg1_p.getElem(10).getDoubleValue();
			SFOC_meas[0][11] = dg1_p.getElem(11).getDoubleValue();
			SFOC_meas[1][0] = dg2_p.getElem(0).getDoubleValue();
			SFOC_meas[1][1] = dg2_p.getElem(1).getDoubleValue();
			SFOC_meas[1][2] = dg2_p.getElem(2).getDoubleValue();
			SFOC_meas[1][3] = dg2_p.getElem(3).getDoubleValue();
			SFOC_meas[1][4] = dg2_p.getElem(4).getDoubleValue();
			SFOC_meas[1][5] = dg2_p.getElem(5).getDoubleValue();
			SFOC_meas[1][6] = dg2_p.getElem(6).getDoubleValue();
			SFOC_meas[1][7] = dg2_p.getElem(7).getDoubleValue();
			SFOC_meas[1][8] = dg2_p.getElem(8).getDoubleValue();
			SFOC_meas[1][9] = dg2_p.getElem(9).getDoubleValue();
			SFOC_meas[1][10] = dg2_p.getElem(10).getDoubleValue();
			SFOC_meas[1][11] = dg2_p.getElem(11).getDoubleValue();	
			SFOC_meas[2][0] = dg3_p.getElem(0).getDoubleValue();
			SFOC_meas[2][1] = dg3_p.getElem(1).getDoubleValue();
			SFOC_meas[2][2] = dg3_p.getElem(2).getDoubleValue();
			SFOC_meas[2][3] = dg3_p.getElem(3).getDoubleValue();
			SFOC_meas[2][4] = dg3_p.getElem(4).getDoubleValue();
			SFOC_meas[2][5] = dg3_p.getElem(5).getDoubleValue();
			SFOC_meas[2][6] = dg3_p.getElem(6).getDoubleValue();
			SFOC_meas[2][7] = dg3_p.getElem(7).getDoubleValue();
			SFOC_meas[2][8] = dg3_p.getElem(8).getDoubleValue();
			SFOC_meas[2][9] = dg3_p.getElem(9).getDoubleValue();
			SFOC_meas[2][10] = dg3_p.getElem(10).getDoubleValue();
			SFOC_meas[2][11] = dg3_p.getElem(11).getDoubleValue();
			SFOC_meas[3][0] = dg3_p.getElem(0).getDoubleValue();
			SFOC_meas[3][1] = dg3_p.getElem(1).getDoubleValue();
			SFOC_meas[3][2] = dg3_p.getElem(2).getDoubleValue();
			SFOC_meas[3][3] = dg3_p.getElem(3).getDoubleValue();
			SFOC_meas[3][4] = dg3_p.getElem(4).getDoubleValue();
			SFOC_meas[3][5] = dg3_p.getElem(5).getDoubleValue();
			SFOC_meas[3][6] = dg3_p.getElem(6).getDoubleValue();
			SFOC_meas[3][7] = dg3_p.getElem(7).getDoubleValue();
			SFOC_meas[3][8] = dg3_p.getElem(8).getDoubleValue();
			SFOC_meas[3][9] = dg3_p.getElem(9).getDoubleValue();
			SFOC_meas[3][10] = dg3_p.getElem(10).getDoubleValue();
			SFOC_meas[3][11] = dg3_p.getElem(11).getDoubleValue();
		}
		
		w_max[0] = dg1_np.getDoubleValue();
		w_max[1] = dg2_np.getDoubleValue();
		w_max[2] = dg3_np.getDoubleValue();
		w_max[3] = dg4_np.getDoubleValue();
		
		svec = new SFOCVector(SFOC_meas);
		
		model = new unbalancedLoadSetPoints(w_max, svec);
		model.setBin_var(gen_bins);
		model.setGen_pris(gen_pris);
		model.setHigh_limit(hl);
		model.setLow_limit(ll);
		model.setDemand(demand.getDoubleValue());
		
		set_points = model.calculateOptimalLoad();
		
		dg1_sp.setDoubleValue(set_points[0]);
		dg1_sp.setFails(0);
		dg2_sp.setDoubleValue(set_points[1]);
		dg2_sp.setFails(0);
		dg3_sp.setDoubleValue(set_points[2]);
		dg3_sp.setFails(0);
		dg4_sp.setDoubleValue(set_points[3]);
		dg4_sp.setFails(0);
		
	} // End of runMe

	/**
	 * moduleupdate 
	 * This method is called once for each prog2 function block
	 * in the module that has been updated using Download to/Update method.
	 */
	public void moduleupdate () {

		// Put update code here.

	} // End of moduleupdate

	/**
	 * exit 
	 * This method is called once for each prog2 function block
	 * using this class. The method is called when the function
	 * block is unloaded from the application server.
	 */
	public void exit() {
		demand = null;
		new_data = null;
		dg1_bin = null;
		dg1_pri = null;
		dg1_hl = null;
		dg1_ll = null;
		dg1_np = null;
		dg1_p = null;
		dg2_bin = null;
		dg2_pri = null;
		dg2_hl = null;
		dg2_ll = null;
		dg2_np = null;
		dg2_p = null;
		dg3_bin = null;
		dg3_pri = null;
		dg3_hl = null;
		dg3_ll = null;
		dg3_np = null;
		dg3_p = null;
		dg4_bin = null;
		dg4_pri = null;
		dg4_hl = null;
		dg4_ll = null;
		dg4_np = null;
		dg4_p = null;
		
		dg1_sp = null;
		dg2_sp = null;
		dg3_sp = null;
		dg4_sp = null;


		System.gc();   // Propose garbage collection.

	} // End of exit

} // End of HVACFlagSetter
